#include "functions.h"

/* ----------------------- *
 * Replacement for strncpy *
 * ----------------------- */

void string_copy(char *dest,char *source,int n)
{
 int i=n;

 while (i>0)
 {
  *(dest+i-1)=*(source+i-1);
  i--;
 }

 *(dest+n)='\0';

}



/* ------------------------------------------------------------ *
 * Function for retrieving a single line (response) from netMod *
 * Returns length of line                                       * 
 * ------------------------------------------------------------ */

int readline(char *s, int fd)
{
 int i=0;
 char c;

 do
 {
  read(fd,&c,1);
  *(s+i)=c;
  i++;
 } while (c!='\n');

 *(s+i)='\0';

 return i;
}



/* --------------------------------------------------- *
 * Initialize netMod device input/output/control modes *
 * Returns 1 on error, or else 0                       *
 * --------------------------------------------------- */
 
int init_netMod(int fd)
{

 struct termios setme;

 setme.c_iflag=IGNBRK;
 setme.c_oflag=NL0|CR0|TAB0|BS0|FF0;
 setme.c_cflag=CREAD|CLOCAL|B115200|CS8|CRTSCTS;
 setme.c_lflag=0;

 if (tcsetattr(fd,TCSANOW,&setme)<0)
 {
  return 1;
 }

 return 0;
}



/* Open Log file in write mode. If it doesn't exist creat it. If it exists *
 * open it in append mode.                                                 *
 * Returns file descriptor of log file, or -1 on error                     */

int open_log()
{
 int logfd=0;
 
 logfd=open(strcat(getenv("HOME"),"/.aoc-log"),
            O_WRONLY|O_CREAT|O_APPEND,
	    0644);
 
 if (logfd<0)
 {
  return -1;
 }

 if ( fchown(logfd,getuid(),getgid())<0 )
 {
  return -1;
 }
 
 return logfd;
}



/* --------------------------------------------------------------------- *
 * Create /var/lock/LCK..(netMod device) to prevent other processes from *
 * accessing netMod                                                      *
 * Returns 1 on error, or else 0                                         *
 * --------------------------------------------------------------------- */

int lock_netMod()
{
 int lock=0;
 char *content;
 
 content=(char *)calloc(15,1);
	 
 lock=open(lockfile,O_WRONLY | O_CREAT | O_EXCL,0644);

 if (lock<0)
 {
  return 1;
 }

 sprintf(content,"%d %s root\n",getpid(),"Cid_Base");
 write(lock,content,strlen(content)+1);

 close(lock);

 return 0;
 
}



/* ----------------------------- *
 * Reset netMod (send ATZ)       *
 * Returns 1 on error, or else 0 *
 * ----------------------------- */
   
int reset_netMod(int fd, char *line)
{

 write(fd,"ATZ\r",4);
 readline(line,fd);
 readline(line,fd);

 if ( (line[0]!='O') && (line[1]!='K') )
 {
  return 1;
 }

 return 0;
 
}



int new_entry(int fd, int log_fd, char *line)
{
 char entry[80];
 time_t now=time(0);
 
 int DTE,POT1,POT2;
 char temp[6];

 write(fd,"ATCH?\r",6);
 readline(line,fd);
 readline(line,fd);
 readline(line,fd);
 readline(line,fd);
 readline(line,fd);
 readline(line,fd);
 readline(line,fd);

 readline(line,fd);
 sscanf(line,"%s %s %d",temp,temp,&DTE);

 readline(line,fd);
 sscanf(line,"%s %s %d",temp,temp,&POT1);
 
 readline(line,fd);
 sscanf(line,"%s %s %d",temp,temp,&POT2);

 sprintf(entry,
         "DTE: %5d\tPOT1: %5d\tPOT2: %5d\tat %s",
	 DTE,POT1,POT2,
	 ctime(&now));

 write(log_fd,entry,strlen(entry));

 readline(line,fd);
 readline(line,fd);
 
 if ( (*line=='O') && (*(line+1)=='K') )
 {
  return 0;
 }
 else
 {
  return 1;
 }


}
